from conippets import json
from chatchat import base
from chatchat.vendor import __vendor_info__

def supported_vendors():
    print(f'Supported vendors:')
    for vendor, attrs in __vendor_info__.items():
        print(vendor)
        for attr in attrs:
            print(f'    {attr}')

def parse_config(args):
    if args.list:
        supported_vendors()
    elif args.cfgs:
        cfg = args.cfgs.split('=')
        vendor_key = cfg[0].split('.')
        usage = 'Usage: chatchat config vendor.key=value'
        if len(cfg) != 2 or len(vendor_key) != 2:
            print(usage)
            return

        (vendor, key), value = vendor_key, cfg[1]
        if vendor not in __vendor_info__:
            print(f'Vendor <{vendor}> is currently NOT supported!')
            supported_vendors()
            return

        secret_file = base.__secret_file__
        secret_data = json.read(secret_file)

        if vendor in secret_data:
            secret_data[vendor][key] = value
        else:
            secret_data[vendor] = {key: value}
        json.write(secret_file, secret_data)

def cli_config(subparser):
    config_parser = subparser.add_parser('config', help='config vendor secret key')
    config_parser.add_argument('cfgs', type=str, nargs='?')
    config_parser.add_argument('--list', action='store_true')
    config_parser.set_defaults(parser=parse_config)



from chatchat.base import Base
from .vendor import __vendor_info__

__vendor__ = 'alibaba'
__vendor_keys__ = __vendor_info__[__vendor__]

class Completion(Base):
    def __init__(self, model='qwen-turbo', client_kwargs={}):
        super().__init__(__vendor__, __vendor_keys__, client_kwargs=client_kwargs)

        self.api_key = self.secret_data[__vendor_keys__[0]]
        self.model = model
        self.api = 'https://dashscope.aliyuncs.com/api/v1/services/aigc/text-generation/generation'
        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.api_key}',
        }

    def send_message(self, messages: list):
        jmsg = {
            'model': self.model,
            "input": {
                "messages": messages,
            },
            "parameters": {
                "result_format": "message"
            }
        }
        r = self.client.post(self.api, headers=self.headers, json=jmsg)
        r = r.json()
        r = self.response(r, ('output', 'choices', 0, 'message', 'content'))
        return r

    def create(self, message):
        jmsg = [{
            "role": "user",
            "content": message,
        }]
        return self.send_message(jmsg)

class Chat(Completion):
    def __init__(self, model='qwen-turbo', history=[], client_kwargs={}):
        super().__init__(model=model, client_kwargs=client_kwargs)
        self.history = history

    def chat(self, message):
        self.history.append({
            'role': 'user',
            'content': message,
        })

        r = self.send_message(self.history)
        assistant_output = r['output']['choices'][0]['message']
        self.history.append(assistant_output)

        return r



import argparse
from chatchat.cli.config import cli_config
from chatchat.cli.chat import cli_chat

parser = argparse.ArgumentParser()
parser.set_defaults(parser=None)
subparser = parser.add_subparsers()

cli_config(subparser)
cli_chat(subparser)

args = parser.parse_args()

def main():
    if args.parser:
        args.parser(args)
    else:
        parser.print_help()

if __name__ == '__main__':
    main()



from chatchat.base import Base
from .vendor import __vendor_info__

__vendor__ = 'deepseek'
__vendor_keys__ = __vendor_info__[__vendor__]

class Completion(Base):
    def __init__(self, model='deepseek-chat', client_kwargs={}):
        super().__init__(__vendor__, __vendor_keys__, client_kwargs=client_kwargs)

        self.api_key = self.secret_data[__vendor_keys__[0]]
        self.model = model
        self.host = 'https://api.deepseek.com'
        self.model_url = f'{self.host}/models'
        self.chat_url = f'{self.host}/chat/completions'
        self.headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'Authorization': f'Bearer {self.api_key}',
        }

    def model_list(self):
        r = self.client.get(self.model_url, headers=self.headers)
        return r.json()

    def send_message(self, message, max_tokens=1024, temperature=1.0, top_p=1.0):
        jmsg = {
            'model': self.model,
            'messages': message,
            'max_tokens': max_tokens,
            'temperature': temperature,
            'top_p': top_p,
        }
        r = self.client.post(self.chat_url, headers=self.headers, json=jmsg)
        r = r.json()
        r = self.response(r, ('choices', 0, 'message', 'content'))
        return r

    def create(self, message, max_tokens=1024, temperature=1.0, top_p=1.0):
        jmsg = [{
            "role": "user",
            "content": message,
        }]
        return self.send_message(jmsg, max_tokens, temperature, top_p)

class Chat(Completion):
    def __init__(self, model='deepseek-chat', history=[], client_kwargs={}):
        super().__init__(model=model, client_kwargs=client_kwargs)
        self.history = history

    def chat(self, message, max_tokens=1024, temperature=1.0, top_p=1.0):
        self.history.append({
            'role': 'user',
            'content': message,
        })
        r = self.send_message(self.history, max_tokens, temperature, top_p)

        if r.text:
            assistant_output = r['choices'][0]['message']
            self.history.append(assistant_output)

        return r



from importlib import import_module
from chatchat.vendor import __vendor_info__

def parse_config(args):
    if args.params:
        vendor, model = args.params

        if vendor not in __vendor_info__:
            print(f'Vendor `{vendor}` is NOT supported!')
            print(f'Current supported vendors:\n    {list(__vendor_info__.keys())}')
            exit(-1)

        module = import_module(f'chatchat.{vendor}')
        chat = module.Chat(model=model, client_kwargs={'proxy': args.proxy})

        while True:
            prompt = input("user> ")
            if prompt == '\x04': # Ctrl+D
                exit()
            response = chat.chat(prompt)
            text = response if response.text is None else response.text
            print(f'assistant> {text}')

def cli_chat(subparser):
    config_parser = subparser.add_parser('run', help='Chat with AI')
    config_parser.add_argument('params', type=str, nargs=2)
    config_parser.add_argument('--proxy', type=str, default=None, required=False)
    config_parser.set_defaults(parser=parse_config)



__vendor_info__ = {
    'alibaba': ('api_key',),
    'baidu': ('app_id',),
    'deepseek': ('api_key',),
    'google': ('api_key',),
    'tencent': ('secret_id', 'secret_key'), # https://console.cloud.tencent.com/cam/capi
    'xunfei': ('api_key', 'service_key'),
    'zhipu': ('api_key',),
}


from chatchat.base import Base
from .vendor import __vendor_info__

__vendor__ = 'xunfei'
__vendor_keys__ = __vendor_info__[__vendor__]

class Completion(Base):
    def __init__(self, model='lite', client_kwargs={}):
        self.model_service = {
            'deepseek-r1': 'xdeepseekr1',
            'deepseek-v3': 'xdeepseekv3',
        }

        if model in self.model_service:
            super().__init__(__vendor__, __vendor_keys__[1:], client_kwargs=client_kwargs)
            # https://training.xfyun.cn/modelService
            self.model = self.model_service[model]
            self.api = 'http://maas-api.cn-huabei-1.xf-yun.com/v1/chat/completions'
            self.api_key = self.secret_data[__vendor_keys__[1]]
        else:
            super().__init__(__vendor__, __vendor_keys__[:1], client_kwargs=client_kwargs)
            self.model = model
            # https://console.xfyun.cn/services/cbm
            self.api_key = self.secret_data[__vendor_keys__[0]]
            self.api = 'https://spark-api-open.xf-yun.com/v1/chat/completions'

        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.api_key}',
        }

    def send_messages(self, messages: list):
        jmsg = {
            'model': self.model,
            "messages": messages,
        }
        url = self.api
        r = self.client.post(url, headers=self.headers, json=jmsg)
        r = r.json()
        r = self.response(r, ('choices', 0, 'message', 'content'))
        return r

    def create(self, message, stream=False):
        messages = [{
            "role": "user",
            "content": message,
        }]
        return self.send_messages(messages)

class Chat(Completion):
    def __init__(self, model='lite', history=[], client_kwargs={}):
        super().__init__(model=model, client_kwargs=client_kwargs)
        self.history = history

    def chat(self, message, stream=False):
        self.history.append({
            "role": "user", "content": message
        })
        r = self.send_messages(self.history)
        if r.text:
            self.history.append(r['choices'][0]['message'])

        return r



from chatchat.base import Base
from .vendor import __vendor_info__

__vendor__ = 'baidu'
__vendor_keys__ = __vendor_info__[__vendor__]

class Completion(Base):
    def __init__(self, model='ernie-speed-8k', client_kwargs={}):
        super().__init__(__vendor__, __vendor_keys__, client_kwargs=client_kwargs)

        self.app_id = self.secret_data[__vendor_keys__[0]]
        self.model = model
        self.api = 'https://qianfan.baidubce.com/v2/chat/completions'
        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.app_id}',
        }

    def send_messages(self, messages):
        jmsg = {
            'model': self.model,
            "messages": messages,
        }
        url = self.api
        r = self.client.post(url, headers=self.headers, json=jmsg)
        r = r.json()
        r = self.response(r, ('choices', 0, 'message', 'content'))
        return r

    def create(self, message):
        messages = [{
            "role": "user",
            "content": message,
        }]
        r = self.send_messages(messages)
        return r

class Chat(Completion):
    def __init__(self, model='ernie-speed-8k', history=[], client_kwargs={}):
        super().__init__(model=model, client_kwargs=client_kwargs)
        self.history = history

    def chat(self, message):
        self.history.append({
            "role": "user",
            "content": message,
        })
        r = self.send_messages(self.history)
        if r.text: self.history.append({"role": "assistant", "content": r.text})
        return r


from chatchat.base import Base
from .vendor import __vendor_info__

__vendor__ = 'google'
__vendor_keys__ = __vendor_info__[__vendor__]

class Completion(Base):
    def __init__(self, model='gemini-2.0-flash', client_kwargs={}):
        super().__init__(__vendor__, __vendor_keys__, client_kwargs=client_kwargs)

        self.api_key = self.secret_data[__vendor_keys__[0]]
        self.model = model
        self.host = 'https://generativelanguage.googleapis.com/v1beta/models'
        self.chat_url = f'{self.host}/{model}:generateContent'
        self.headers = {'Content-Type': 'application/json'}
        self.params = {'key': self.api_key}

    def message_template(self, role, text):
        message = {
            "role": role,
            "parts": [{"text": text}],
        }
        return message

    def send_message(self, message):
        jmsg = {'contents': message}
        r = self.client.post(self.chat_url, headers=self.headers, json=jmsg, params=self.params)
        r = r.json()
        r = self.response(r, ('candidates', 0, 'content', 'parts', 0, 'text'))
        return r

    def create(self, message):
        msg = [self.message_template('user', message)]
        return self.send_message(msg)

class Chat(Completion):
    def __init__(self, model='gemini-2.0-flash', history=[], client_kwargs={}):
        super().__init__(model=model, client_kwargs=client_kwargs)
        self.history = history

    def chat(self, message):
        self.history.append(self.message_template('user', message))
        r = self.send_message(self.history)

        if r.text:
            assistant_output = r['candidates'][0]['content']
            self.history.append(assistant_output)

        return r



from chatchat.base import Base
import hashlib, hmac, json, time
from datetime import datetime
from .vendor import __vendor_info__

__vendor__ = 'tencent'
__vendor_keys__ = __vendor_info__[__vendor__]

class Completion(Base):
    def __init__(self, model='hunyuan-lite', client_kwargs={}):
        super().__init__(__vendor__, __vendor_keys__, client_kwargs=client_kwargs)

        self.secret_id = self.secret_data[__vendor_keys__[0]]
        self.secret_key = self.secret_data[__vendor_keys__[1]]
        self.model = model
        self.host = 'cvm.tencentcloudapi.com'
        self.endpoint = f'https://{self.host}'

    def encode_message(self, jmsg):
        # step 1
        http_request_method = "POST"
        canonical_uri = "/"
        canonical_querystring = ""
        ct = "application/json; charset=utf-8"
        action = 'ChatCompletions'
        host = 'hunyuan.tencentcloudapi.com'
        payload = json.dumps(jmsg)
        canonical_headers = "content-type:%s\nhost:%s\nx-tc-action:%s\n" % (ct, host, action.lower())
        signed_headers = "content-type;host;x-tc-action"
        hashed_request_payload = hashlib.sha256(payload.encode("utf-8")).hexdigest()
        canonical_request = (http_request_method + "\n" +
                            canonical_uri + "\n" +
                            canonical_querystring + "\n" +
                            canonical_headers + "\n" +
                            signed_headers + "\n" +
                            hashed_request_payload)

        # step 2
        service = "cvm"
        algorithm = "TC3-HMAC-SHA256"
        timestamp = int(time.time())
        date = datetime.utcfromtimestamp(timestamp).strftime("%Y-%m-%d")
        credential_scope = date + "/" + service + "/" + "tc3_request"
        hashed_canonical_request = hashlib.sha256(canonical_request.encode("utf-8")).hexdigest()
        string_to_sign = (algorithm + "\n" +
                        str(timestamp) + "\n" +
                        credential_scope + "\n" +
                        hashed_canonical_request)

        # step 3
        def sign(key, msg):
            return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()

        secret_date = sign(("TC3" + self.secret_key).encode("utf-8"), date)
        secret_service = sign(secret_date, service)
        secret_signing = sign(secret_service, "tc3_request")
        signature = hmac.new(secret_signing, string_to_sign.encode("utf-8"), hashlib.sha256).hexdigest()

        # step 4
        authorization = (algorithm + " " +
                        "Credential=" + self.secret_id + "/" + credential_scope + ", " +
                        "SignedHeaders=" + signed_headers + ", " +
                        "Signature=" + signature)

        headers = {
            'Authorization': authorization,
            'Content-Type': 'application/json; charset=utf-8',
            'Host': self.host,
            'X-TC-Action': 'ChatCompletions',
            'X-TC-Timestamp': str(timestamp),
            'X-TC-Version': '2023-09-01',
            'X-TC-Region': 'ap-beijing',
        }

        return headers, payload

    def send_message(self, history: list):
        jmsg = {
            "TopP": 1,
            "Temperature": 1,
            "Model": self.model,
            "Messages": history,
            "Stream": False,
        }
        headers, payload = self.encode_message(jmsg)
        r = self.client.post(self.endpoint, headers=headers, data=payload)
        r = r.json()
        r = self.response(r, ('Response', 'Choices', 0, 'Message', 'Content'))
        return r

    def create(self, message, stream=False):
        jmsg = [{
            "Role": "user",
            "Content": message,
        }]
        return self.send_message(jmsg)

class Chat(Completion):
    def __init__(self, model='hunyuan-lite', history=[], client_kwargs={}):
        super().__init__(model=model, client_kwargs=client_kwargs)
        self.history = history

    def chat(self, message):
        self.history.append({
            'Role': 'user',
            'Content': message,
        })

        r = self.send_message(self.history)
        if r.text:
            assistant_output = r['Response']['Choices'][0]['Message']
            self.history.append(assistant_output)

        return r



from setuptools import setup, find_packages
from chatchat.version import __version__

setup(
    name = 'chatchat',
    packages = find_packages(exclude=['examples']),
    version = __version__,
    license = 'GPL-2.0',
    description = 'Large Language Models Python API ',
    author = 'JiauZhang',
    author_email = 'jiauzhang@163.com',
    url = 'https://github.com/JiauZhang/chatchat',
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type = 'text/markdown',
    keywords = [
        'llm',
        'chatapi',
        'chatbot',
    ],
    install_requires=[
        'conippets >= 0.1.1', 'httpx',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Programming Language :: Python :: 3.8',
    ],
    entry_points = {
        'console_scripts': ['chatchat=chatchat.__main__:main'],
    },
)


import pathlib, os, httpx
from conippets import json
from functools import cached_property

__secret_file__ = os.path.join(str(pathlib.Path.home()), '.chatchat.json')

class Response(dict):
    def __init__(self, raw_response, text_keys):
        super().__init__(**raw_response)
        self.text_keys = text_keys

    @cached_property
    def text(self):
        text = self
        for key in self.text_keys:
            try:
                text = text[key]
            except:
                text = None
                break
        return text

class Base():
    def __init__(self, vendor, vendor_keys, client_kwargs={}):
        self.vendor = vendor
        if not os.path.exists(__secret_file__):
            json.write(__secret_file__, {})

        secret_data = json.read(__secret_file__)
        self.verify_secret_data(secret_data, self.vendor, vendor_keys)
        self.secret_file = __secret_file__
        self.secret_data = secret_data[self.vendor]
        self.client = httpx.Client(**client_kwargs)

    def verify_secret_data(self, secret_data, vendor, vendor_keys):
        has_vendor = vendor in secret_data
        has_key = True
        if has_vendor:
            vendor_data = secret_data[vendor]
            for key in vendor_keys:
                has_key = has_key and key in vendor_data
        if not (has_vendor and has_key):
            print('You need to configure your target vendor first as bellow.')
            for key in vendor_keys:
                print(f'    chatchat config {vendor}.{key}=YOUR_{key.upper()}')
            exit(-1)

    def response(self, raw_response, text_keys):
        return Response(raw_response, text_keys)